/*
	PROforma, PROgs FOnt & Raster MAnager

	Black drawing routines for mode 4 bitmap driver

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 28 September 1995
*/

#include "PFmodule.h"

Error BlackPixel(Bitmap bitmap, short x, short y)
{
	unsigned char *base=bitmap->base;	/* base of bitmap */
	char xb;						/* bit pattern for x */

	/* clip pixel to useable area */
	if (x<0 || x>=bitmap->clip.xsiz ||
		y<0 || y>=bitmap->clip.ysiz)
		return ERR_OK;
	x+=bitmap->clip.xorg;			/* add origin in bitmap !! */
	y+=bitmap->clip.yorg;

	xb=~( 0x80 >> (x&(short)7) );	/* mask to clear the pixel */
	base+=y*bitmap->linc+((x>>2)&(-2)); /* address in bitmap to set pixels */

	*base++&=xb;
	*base  &=xb;
	return ERR_OK;
}

Error BlackLineHorizontal(Bitmap bitmap, short y, short xs, short xe)
{
	unsigned char *base=bitmap->base;	/* base of bitmap */
	char xl, xr;					/* bit pattern for x */
	short dist; 					/* number of pixels to fill */

	/* clip line to useable area */
	if (xe<0 || xs>=bitmap->clip.xsiz ||
		 y<0 ||  y>=bitmap->clip.ysiz)
		return ERR_OK;
	if (xs<0) xs=0;
	if (xe>=bitmap->clip.xsiz) xe=bitmap->clip.xsiz;
	xs+=bitmap->clip.xorg;			/* add origin in bitmap !! */
	xe+=bitmap->clip.xorg;
	y +=bitmap->clip.yorg;
										
	dist=xe-xs;
	if (dist<=0) return ERR_OK; 	/* we need a positive length */

	xl=~(0xff >> (xs&7));			/* mask for pixels at left */
	xr= (0xff >> (xe&7));			/* mask for pixels at right */
	base+=y*bitmap->linc+((xs>>2)&(-2));/* address in bitmap to set pixels */

	dist-=8-(xs&7); 				/* no of pixels to go after first byte */

	if (dist<0) 					/* only one byte to adjust */
	{
		xl|=xr;
		*base++&=xl;
		*base  &=xl;
	} else {                        /* adjust at least two bytes */
		*base++&=xl;
		*base++&=xl;
		for (dist>>=3 ; dist ; dist--) *base++=0, *base++=0;
		*base++&=xr;
		*base  &=xr;
	}
	return ERR_OK;
}
