/*
	PROforma, PROGS Font & Raster Manager

	DeskJet CMY Colour printer driver

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started April 9 1997, based on DeskJet 500 driver
*/

#include <mem.h>
#include "driv_pcl_h"

Bbox PrintableArea={
	short2pt(576), short2pt(800),	/* size */
	short2pt(7.2), short2pt(4.56)	/* origin */
};

char DefaultDevice[IO_MAXFULLNAME]="pard";

Error Define(char *name, char *value)
{
	double x,y;

	if (STRSameCD(name,"DEFAULT-DEVICE"))
	{
		STRCopySafe(value,DefaultDevice,IO_MAXFULLNAME);
		return ERR_OK;
	}
	if (STRSameCD(name,"PRINTABLE-AREA-SIZE"))
	{
		STRFormatGet(value,NULL,"%f %f",&x,&y);
		PrintableArea.xsiz=double2pt(x);
		PrintableArea.ysiz=double2pt(y);
		return ERR_OK;
	}
	if (STRSameCD(name,"PRINTABLE-AREA-ORIGIN"))
	{
		STRFormatGet(value,NULL,"%f %f",&x,&y);
		PrintableArea.xorg=double2pt(x);
		PrintableArea.yorg=double2pt(y);
		return ERR_OK;
	}
	return ERR_IPAR;
}

Error InitInit(Gstate gstate, char *device, PBbox *area)
{
	Error err;
	DriverInfo *di;
	DRIVdriver *prt=gstate->printer;

	err=MEMAllocate(sizeof(DriverInfo),&di);
	if (err) return err;

	di->type=LEVEL5+NORESET;		/* this is a DeskJet driver */
	di->copies=1;					/* start with one copy a page */

	if (device && *device)
		err=IOOpen(device,OPEN_OVER,&di->channel);
	else
		err=IOOpen(DefaultDevice,OPEN_OVER,&di->channel);
	if (err) { MEMRelease(di); return err; };

	gstate->PrinterExtra=di;
	gstate->todev.x = pt_one * prt->xdpi / 72;
	gstate->todev.y = pt_one * prt->ydpi / 72;
	gstate->ppp.x = pt_one * 72 / prt->xdpi;
	gstate->ppp.y = pt_one * 72 / prt->ydpi;

	area->xsiz=pt2rshort(fixmul(PrintableArea.xsiz,gstate->todev.x));
	area->ysiz=pt2rshort(fixmul(PrintableArea.ysiz,gstate->todev.y));
	area->xorg=pt2rshort(fixmul(PrintableArea.xorg,gstate->todev.x));
	area->yorg=pt2rshort(fixmul(PrintableArea.yorg,gstate->todev.y));

	return err;
}

Error InitDone(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	Size linc=gstate->raster->linc*2;

	/* complete the initialisation of the DriverInfo */
	di->out_line =(char *)gstate->raster->base;
	di->prev_line=di->out_line+linc;
	di->alt_line =di->prev_line+linc;
	gstate->raster->base=(unsigned char *)(di->alt_line+linc);

	di->xskip=pt2rshort(gstate->pagebbox.xporg-fixmul(PrintableArea.xorg,
							   gstate->todev.x));
	di->yskip=pt2rshort(gstate->pagebbox.yporg-fixmul(PrintableArea.yorg,
							   gstate->todev.y));

	return ERR_OK;
}

Error BufferSize(Gstate gstate, int xsiz, int ysiz, Size *size)
{
	return gstate->bitmap->BufferSize(xsiz,ysiz+6,size);
}

Error CleanUp(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;

	if (di)
	{
		IOClose(di->channel);
		MEMRelease(gstate->PrinterExtra);
	}
	return ERR_OK;
}

Error ColourAdjust(Gstate gstate, ColourCMYK *colour)
{
	colour->cyan   +=colour->black;
	colour->magenta+=colour->black;
	colour->yellow +=colour->black;
	newmax(colour->cyan   ,pt_hundred);
	newmax(colour->magenta,pt_hundred);
	newmax(colour->yellow ,pt_hundred);
	colour->black=0;
	return ERR_OK;
}

Error ColourNearest(Gstate gstate, ColourCMYK *colour, int *result)
{
	*result=0;						/* assume white */
	if (colour->cyan>pt_hundred/2)
		*result+=4;
	if (colour->magenta>pt_hundred/2)
		*result+=2;
	if (colour->yellow>pt_hundred/2)
		*result+=1;
	return ERR_OK;
}

Error ColourDelta(Gstate gstate, ColourCMYK *colour, ColourCMYK *delta)
{
	*delta=*colour;
	if (colour->cyan>pt_hundred/2)
		delta->cyan-=pt_hundred;
	if (colour->magenta>pt_hundred/2)
		delta->magenta-=pt_hundred;
	if (colour->yellow>pt_hundred/2)
		delta->yellow-=pt_hundred;
	return ERR_OK;
}

static Error nimp() 	{ return ERR_NIMP; }
static Error nothing()	{ return ERR_OK; }

Error InitDriver();

Error SetCopies(Gstate, int count);
Error SetDisplayMode(Gstate, int mode);
Error WindowMove(Gstate, int dx, int dy);
Error BufferUpdate(Gstate);
Error InitPage(Gstate);
Error ShowPage(Gstate);
Error StopPage(Gstate);
Error GetFullPageBbox(Gstate, Bbox *box);
Error DefaultDev(Gstate, char *device);

#define InitDriver		nothing
#define WindowMove		nothing
#define SetDisplayMode	nimp
#define GetFullPageBbox nimp
#define DefaultDev		nimp
#define SetCopies		nimp
#define BufferUpdate	(Error (*)(Gstate))nimp
#define BufferSave		(Error (*)(Gstate))nimp
#define BufferRestore	(Error (*)(Gstate))nimp
#define Handle			nimp

DRIVdriver dpi100={
	NULL, PF_PRINTERDRIVER,
	"HP DeskJet CMY Colour 100dpi",    /* name */
	100,100,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		3, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_SEPARATE_PLANES, /* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_CMY,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDev,

	Handle
};

DRIVdriver dpi150={
	&dpi100, PF_PRINTERDRIVER,
	"HP DeskJet CMY Colour 150dpi",    /* name */
	150,150,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		3, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_SEPARATE_PLANES, /* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_CMY,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDev,

	Handle
};

DRIVdriver init={
	&dpi150, PF_PRINTERDRIVER,
	"HP DeskJet CMY Colour 300dpi",    /* name */
	300,300,					/* xdpi, ydpi */
	{                           /* mode 4 bitmap */
		3, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_SEPARATE_PLANES, /* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_CMY,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDev,

	Handle
};
